// derived from malloc_allocator.h  (GNU C++ library)

// Allocator that wraps "C" malloc -*- C++ -*-
// and shreds memory before calling free

// Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License along
// with this library; see the file COPYING.  If not, write to the Free
// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.

// As a special exception, you may use this file as part of a free software
// library without restriction.  Specifically, if other files instantiate
// templates or use macros or inline functions from this file, or you compile
// this file and link it with other files to produce an executable, this
// file does not by itself cause the resulting executable to be covered by
// the GNU General Public License.  This exception does not however
// invalidate any other reasons why the executable file might be covered by
// the GNU General Public License.

#include <stddef.h>
#ifndef __CONGA_SHRED_ALLOCATOR_H
#define __CONGA_SHRED_ALLOCATOR_H

extern "C" {
	#include <unistd.h>
	#include <stdlib.h>
	#include <string.h>
}

#include <new>

template<typename _Tp>
void
shred(_Tp* __p, size_t n)
{
	size_t size = sizeof(_Tp) / sizeof(char) * n;
	if (size && __p)
		memset(__p, 0, size);
}

template<typename _Tp>
class shred_allocator
{
public:
  typedef size_t     size_type;
  typedef ptrdiff_t  difference_type;
  typedef _Tp*       pointer;
  typedef const _Tp* const_pointer;
  typedef _Tp&       reference;
  typedef const _Tp& const_reference;
  typedef _Tp        value_type;

  template<typename _Tp1>
  struct rebind
  { typedef shred_allocator<_Tp1> other; };

  shred_allocator() throw() { }

  shred_allocator(const shred_allocator&) throw() { }

  template<typename _Tp1>
  shred_allocator(const shred_allocator<_Tp1>&) throw() { }

  ~shred_allocator() throw() { }

  pointer
  address(reference __x) const { return &__x; }

  const_pointer
  address(const_reference __x) const { return &__x; }

  // NB: __n is permitted to be 0.  The C++ standard says nothing
  // about what the return value is when __n == 0.
  pointer
  allocate(size_type __n, const void* = 0)
  {
    pointer __ret = static_cast<_Tp*>(malloc(__n * sizeof(_Tp)));
    if (!__ret)
      throw std::bad_alloc();
    return __ret;
  }

  // __p is not permitted to be a null pointer.
  void
  deallocate(pointer __p, size_type n)
  {
    shred(__p, n);
    free(static_cast<void*>(__p));
  }

  size_type
  max_size() const throw()
  { return size_t(-1) / sizeof(_Tp); }

  // _GLIBCXX_RESOLVE_LIB_DEFECTS
  // 402. wrong new expression in [some_] allocator::construct
  void
  construct(pointer __p, const _Tp& __val)
  { ::new(__p) _Tp(__val); }

  void
  destroy(pointer __p) { __p->~_Tp(); }
};

template<typename _Tp>
inline bool
operator==(const shred_allocator<_Tp>&, const shred_allocator<_Tp>&)
{ return true; }

template<typename _Tp>
inline bool
operator!=(const shred_allocator<_Tp>&, const shred_allocator<_Tp>&)
{ return false; }


#endif
